@@ -12,8 +12,14 @@
 #include <sys/types.h>
 
 #include <gtkmm.h>
+#include <gtk/gtk.h>
 #include <sigc++/bind.h>
 
+#if defined(__linux)
+#include <X11/Xlib.h>
+#include <gdk/gdkx.h>
+#endif
+
 using namespace Gtk;
 
 #include "../AudioOutput.h"
@@ -21,10 +27,17 @@ using namespace Gtk;
 #include "../Preset.h"
 #include "../VoiceAllocationUnit.h"
 
-#include "EditorPanel.h"
 #include "../../config.h"
 #include "amsynth_logo.h"
 #include "ConfigDialog.h"
+#include "MIDILearnDialog.h"
+
+#include "gui_main.h"
+#include "editor_pane.h"
+
+#ifndef DEFAULT_PREFIX
+#define DEFAULT_PREFIX "/usr/local"
+#endif
 
 enum {
 	evLoad,
@@ -42,13 +55,24 @@ enum {
 	evVkeybd,
 	evMidiSend,
 	evConfig,
+	evNewInstance,
+	
+	evMax
 };
 
+static MIDILearnDialog *g_midiLearn = NULL;
+
+void modal_midi_learn(int param_index) // called by editor_pane upon right-clicking a control
+{
+	if (g_midiLearn)
+		g_midiLearn->run_modal(param_index);
+}
+
 int
 GUI::delete_event_impl(GdkEventAny *)
 {
 	if (m_presetIsNotSaved) {
-		MessageDialog dlg (*this, "Really quit amSynth?\n\nYou will lose any changes\nwhich you haven't explicitly commited", false, MESSAGE_QUESTION, BUTTONS_YES_NO, true);
+		MessageDialog dlg (*this, "Really quit amSynth?\n\nYou will lose any changes\nwhich you haven't explicitly committed", false, MESSAGE_QUESTION, BUTTONS_YES_NO, true);
 		if (RESPONSE_YES != dlg.run())
 			return false;
 	}
@@ -56,29 +80,31 @@ GUI::delete_event_impl(GdkEventAny *)
 	return true;
 }
 
-GUI::GUI( Config & config, MidiController & mc, VoiceAllocationUnit & vau,
+GUI::GUI( Config & config_in, MidiController & mc, VoiceAllocationUnit & vau_in,
 		  GenericOutput *audio, const char *title )
-:	controller_map_dialog(NULL)
-,	clipboard_preset (new Preset)
+#if ENABLE_MIDIKEYS
+:	m_vkeybdOctave(4)
+,	m_vkeybdIsActive(false)
+,	m_vkeybdState(128)
+#endif
 {
 	lnav = -1;
 	
-	this->config = &config;
+	this->config = &config_in;
 	this->midi_controller = &mc;
-	this->vau = &vau;
+	this->vau = &vau_in;
 	this->audio_out = audio;
 	
 //	if(this->config->realtime)
 		// messes up the audio thread's timing if not realtime...
-//		Gtk::Main::timeout.connect( mem_fun(*this,&GUI::idle_callback), 200 );
+//		Glib::signal_timeout().connect( mem_fun(*this,&GUI::idle_callback), 200 );
 
         m_baseName = title + string(" : ");
 	set_title(m_baseName);
 	set_resizable(false);
         
 	active_param = 0;
-
-
+	
 //	style = Gtk::Style::create ( );
 	
 	
@@ -147,11 +173,7 @@ GUI::GUI( Config & config, MidiController & mc, VoiceAllocationUnit & vau,
 	about_authors.push_back("Martin Tarenskeen");
 	about_authors.push_back("Adrian Knoth");
 	about_authors.push_back("Samuli Suominen");
-	aboutDlg.set_authors(about_authors);
-	std::list<std::string> about_artists;
-	about_artists.push_back("Saul Cross");
-	aboutDlg.set_artists(about_artists);
-	
+	aboutDlg.set_authors(about_authors);	
 	
 	//
 	// the record dialog
@@ -187,6 +209,8 @@ GUI::GUI( Config & config, MidiController & mc, VoiceAllocationUnit & vau,
 	
 	record_recording = false;
 	record_statusbar.push ("capture status: STOPPED", 1);
+
+	gtk_window_set_icon_from_file(this->gobj(), DEFAULT_PREFIX "/share/pixmaps/amsynth.png", NULL);
 }
 
 Gtk::MenuBar*
@@ -201,12 +225,21 @@ GUI::create_menus	( )
 	//
 	Menu *menu_file = manage (new Menu());
 	MenuList& list_file = menu_file->items ();
-	list_file.push_back (MenuElem("New Instance", sigc::bind(mem_fun(*this, &GUI::command_run),"amSynth")));
+	
+#if defined(__linux)
+	// create-new-instance currently only supported on Linux
+	list_file.push_back (MenuElem("New Instance", sigc::bind(mem_fun(this, &GUI::event_handler),(int)evNewInstance)));
 	list_file.push_back (SeparatorElem());
+#endif
+	
 	list_file.push_back (MenuElem("_Open Bank",Gtk::AccelKey("<control>O"), mem_fun(*this, &GUI::bank_open)));
 //	list_file.push_back (MenuElem("_Save Bank","<control>S", mem_fun(*this, &GUI::bank_save)));
 	list_file.push_back (MenuElem("_Save Bank As...",Gtk::AccelKey("<control>S"), mem_fun(*this, &GUI::bank_save_as)));
 	list_file.push_back (SeparatorElem());
+	list_file.push_back (MenuElem("Open Alternate Tuning File", mem_fun(*this, &GUI::scale_open)));
+	list_file.push_back (MenuElem("Open Alternate Keyboard Map", mem_fun(*this, &GUI::key_map_open)));
+	list_file.push_back (MenuElem("Reset All Tuning Settings to Default", mem_fun(*this, &GUI::tuning_reset)));
+	list_file.push_back (SeparatorElem());
 	list_file.push_back (MenuElem("_Quit",Gtk::AccelKey("<control>Q"), bind(mem_fun(this, &GUI::event_handler),(int)evQuit)));
 	
 	
@@ -235,8 +268,56 @@ GUI::create_menus	( )
 	//
 	Menu *menu_config = manage (new Menu());
 	MenuList& list_config = menu_config->items ();
+
+	//
+	// Config > MIDI Channel
+	//
+	{
+		Gtk::RadioButtonGroup grp;
+		Gtk::RadioMenuItem *item = NULL;
+		Gtk::Menu *menu = Gtk::manage( new Gtk::Menu );
+		const int currentValue = midi_controller->get_midi_channel();
+		
+		for (int i=0; i<=16; i++) {
+			ostringstream name; i ? name << i : name << "All";
+			item = Gtk::manage(new Gtk::RadioMenuItem(grp, name.str()));
+			item->set_active((i == currentValue));
+			menu->items().push_back(*item);
+		}
+		
+		// connect the signal handlers after calling set_active to prevent altering the value.
+		for (int i=0; i<16; i++) {
+			menu->items()[i].signal_activate().connect( sigc::bind(mem_fun(*this, &GUI::on_midi_channel_change), i) );
+		}
+
+		menu_config->items().push_back(MenuElem("MIDI Channel", *menu));
+	}
+	
+	//
+	// Config > Polyphony
+	//
+	{
+		Gtk::RadioButtonGroup grp;
+		Gtk::RadioMenuItem *item = NULL;
+		Gtk::Menu *menu = Gtk::manage( new Gtk::Menu );
+		const int currentValue = vau->GetMaxVoices();
+		
+		item = Gtk::manage(new Gtk::RadioMenuItem(grp, "Unlimited"));
+		item->signal_activate().connect( sigc::bind(mem_fun(*this, &GUI::on_ployphony_change), 0) );
+		menu->items().push_back(*item);
+		
+		for (int i=1; i<=16; i++) {
+			ostringstream name; name << i;
+			Gtk::RadioMenuItem *item = Gtk::manage(new Gtk::RadioMenuItem(grp, name.str()));
+			item->set_active((i == currentValue));
+			item->signal_activate().connect( sigc::bind(mem_fun(*this, &GUI::on_ployphony_change), i) );
+			menu->items().push_back(*item);
+		}
+		
+		menu_config->items().push_back(MenuElem("Max. Polyphony", *menu));
+	}
+	
 	list_config.push_back (MenuElem("Audio & MIDI...", bind(mem_fun(*this, &GUI::event_handler), (int)evConfig)));
-	list_config.push_back (MenuElem("MIDI Controllers...", Gtk::AccelKey("<control>M"), mem_fun(*this, &GUI::config_controllers)));
 	
 	
 	//
@@ -325,43 +406,39 @@ GUI::create_menus	( )
 
 
 void
-GUI::config_controllers()
+adjustment_value_changed (GtkAdjustment *adjustment, gpointer data)
 {
-	if (controller_map_dialog) controller_map_dialog->show_all();
+	Parameter *param = (Parameter *)data;
+	param->setValue (gtk_adjustment_get_value (adjustment));
 }
 
-
-
 void 
 GUI::init()
 {
 	Preset *preset = &(preset_controller->getCurrentPreset());
 	
-	editor_panel = new EditorPanel (preset);
 	
-	adj_midi = manage (new Gtk::Adjustment(config->midi_channel,0,16,1));
-	adj_midi->signal_value_changed().connect (mem_fun (*this, &GUI::changed_midi_channel));
-	Gtk::SpinButton *sb_midi = manage (new Gtk::SpinButton(*adj_midi,1,0));
+	for (int i=0; i<kControls_End; i++) {
+		Parameter &param = preset->getParameter(i);
+		m_adjustments[i] = (GtkAdjustment *) gtk_adjustment_new (
+			param.getValue(),
+			param.getMin(),
+			param.getMax(),
+			param.getStep(),
+			0, 0);
+		gtk_signal_connect (GTK_OBJECT (m_adjustments[i]), "value_changed",
+			(GtkSignalFunc) adjustment_value_changed,
+			(gpointer) &param );
+	}
 	
-	adj_voices = manage (new Gtk::Adjustment(config->polyphony,1,128,1));
-	adj_voices->signal_value_changed().connect (mem_fun (*this, &GUI::changed_voices));
-	Gtk::SpinButton *sb_voices = manage (new Gtk::SpinButton(*adj_voices,1,0));
+	Gtk::Widget *editor = Glib::wrap (editor_pane_new (m_adjustments));
 	
 	vbox.pack_start (*(create_menus ()),0,0);
 	Gtk::HBox *tmphbox = manage (new Gtk::HBox());
 	tmphbox->pack_start(*presetCV,0,0);
-	Gtk::HBox *midibox = manage(new Gtk::HBox());
-	midibox->pack_start(*(manage( new Gtk::Label (" midi ch:") )),0,0);
-	midibox->pack_start(*sb_midi,0,0);
-	midibox->pack_start(*(manage( new Gtk::Label (" poly:") )),0,0);
-	midibox->pack_start(*sb_voices,0,0);
-	Gtk::Alignment *align = manage(new Gtk::Alignment(Gtk::ALIGN_RIGHT,
-							  Gtk::ALIGN_CENTER,
-							  0,0));
-	align->add(*midibox);
-	tmphbox->pack_start(*align);
+
 	vbox.pack_start (*tmphbox,0,0);
-	vbox.pack_start (*editor_panel,Gtk::PACK_EXPAND_WIDGET,0);
+	vbox.pack_start (*editor, Gtk::PACK_EXPAND_WIDGET,0);
 	vbox.pack_start (statusBar,PACK_SHRINK);
 	add(vbox);
 	
@@ -385,9 +462,9 @@ GUI::init()
 	
 	statusBar.pack_start (*manage(new Gtk::VSeparator), PACK_SHRINK);
 	
-	static char cstr[32];
-	sprintf( cstr, "Sample Rate: %d", config->sample_rate );
-	statusBar.pack_start (*manage(new Gtk::Label (cstr)), PACK_SHRINK, padding);
+	ostringstream oss;
+	oss << "Sample Rate: " << config->sample_rate;
+	statusBar.pack_start (*manage(new Gtk::Label (oss.str())), PACK_SHRINK, padding);
 
 	statusBar.pack_start (*manage(new Gtk::VSeparator), PACK_SHRINK);
 	
@@ -399,7 +476,7 @@ GUI::init()
 		statusBar.pack_start(*manage(new Gtk::Label (status)), PACK_SHRINK, padding);
 	}
 #endif
-	
+
 	show_all();
 	
 	//
@@ -456,7 +533,6 @@ GUI::post_init()
 #endif
 }
 
-
 void 
 GUI::event_handler(const int e)
 {
@@ -581,6 +657,10 @@ GUI::event_handler(const int e)
 		dlg.run ();
 		break;
 	}
+	
+	case evNewInstance:
+		spawn_new_instance();
+		break;
 		
 	default:
 		cout << "no handler for event: " << e << endl;
@@ -593,27 +673,21 @@ GUI::idle_callback()
 {
 	if (vau->GetActiveVoices() != lnav)
 	{
-		string txt = status;
-		txt += "   ";
-		char cstr[3];
-		sprintf (cstr, "%d", vau->GetActiveVoices());
-		txt += string( cstr );
+		ostringstream txt;
+		txt << status << "   " << vau->GetActiveVoices();
 		if( config->polyphony != 0 )
 		{
-			sprintf( cstr, "%d", config->polyphony );
-			txt += "/";
-			txt += string(cstr);
+			txt << "/" << config->polyphony;
 		}
-		txt += " Voices Active";
+		txt << " Voices Active";
 		statusBar.pop( 1 );
-		statusBar.push (txt, 1);
+		statusBar.push (txt.str(), 1);
 	}
 	return true;
 }
 
 GUI::~GUI()
 {
-	delete controller_map_dialog;
 }
 
 void
@@ -637,6 +711,11 @@ GUI::update_title()
 	if (m_presetIsNotSaved) {
 		title += std::string(" *");
 	}
+#if ENABLE_MIDIKEYS
+	if (m_vkeybdIsActive) {
+		title += std::string(" (midikeys active)");
+	}
+#endif
 	set_title(title);
 }
 
@@ -654,8 +733,20 @@ GUI::UpdateParameter(Param paramID, float paramValue)
 }
 
 void
-GUI::UpdateParameterOnMainThread(Param, float)	// called whenever a parameter value has changed
+GUI::UpdateParameterOnMainThread(Param paramID, float)	// called whenever a parameter value has changed
 {
+	if (0 <= paramID && paramID < kControls_End)
+	{
+		const Parameter &param = preset_controller->getCurrentPreset().getParameter(paramID);
+	
+		gtk_adjustment_set_value (m_adjustments[paramID], param.getValue());
+	
+		char tmpstr[32] = "";
+		snprintf (tmpstr, 32, "%s = %.3f", param.getName().c_str(), param.getControlValue());
+		statusBar.pop (0);
+		statusBar.push (Glib::ustring (tmpstr), 0);
+	}
+	
 	bool isModified = preset_controller->isCurrentPresetModified();
 	if (m_presetIsNotSaved != isModified) {
 		m_presetIsNotSaved = isModified;
@@ -688,7 +779,7 @@ GUI::setPresetController(PresetController & p_c)
 		preset.getParameter(i).addUpdateListener(*this);
 	}
 	
-    controller_map_dialog = new ControllerMapDialog(midi_controller, preset_controller);
+	g_midiLearn = new MIDILearnDialog(midi_controller, preset_controller, this->gobj());
 }
 
 void
@@ -697,25 +788,55 @@ GUI::preset_new		( )
 	preset_controller->newPreset ();
 }
 
+////////////////////////////////////////////////////////////////////////////////
+
 void
 GUI::preset_copy	( )
 {
-	*clipboard_preset = preset_controller->getCurrentPreset ();
+	std::string presetData = preset_controller->getCurrentPreset().toString();
+	gtk_clipboard_set_text(gtk_clipboard_get(GDK_SELECTION_CLIPBOARD), presetData.c_str(), -1);
 }
 
 void
 GUI::preset_paste	( )
 {
-	preset_controller->getCurrentPreset() = *clipboard_preset;
+	gtk_clipboard_request_text(gtk_clipboard_get(GDK_SELECTION_CLIPBOARD), GUI::preset_paste_callback, this);
+}
+
+void
+GUI::preset_paste_callback(GtkClipboard *, const gchar *text, gpointer userdata)
+{
+	GUI *_this = reinterpret_cast<GUI *>(userdata);
+	
+	Preset pastedPreset;
+	if (!pastedPreset.fromString(std::string(text)))
+		return;
+	
+	// enure preset has a unique name
+	for (int suffixNumber = 1; _this->preset_controller->containsPresetWithName(pastedPreset.getName()); suffixNumber++) {
+		std::stringstream str; str <<  pastedPreset.getName() << " " << suffixNumber;
+		pastedPreset.setName(str.str());
+	}
+	
+	_this->preset_controller->getCurrentPreset() = pastedPreset;
+	_this->presetCV->update();
 }
 
 void
 GUI::preset_paste_as_new( )
 {
-	preset_new ();
-	preset_controller->getCurrentPreset() = *clipboard_preset;
+	gtk_clipboard_request_text(gtk_clipboard_get(GDK_SELECTION_CLIPBOARD), GUI::preset_paste_as_new_callback, this);
 }
 
+void
+GUI::preset_paste_as_new_callback(GtkClipboard *clipboard, const gchar *text, gpointer userdata)
+{
+	reinterpret_cast<GUI *>(userdata)->preset_new ();
+	preset_paste_callback(clipboard, text, userdata);
+}
+
+////////////////////////////////////////////////////////////////////////////////
+
 void
 GUI::bank_open		( )
 {
@@ -744,6 +865,67 @@ GUI::bank_save_as	( )
 	}
 }
 
+void
+GUI::scale_open		( )
+{
+	FileChooserDialog dlg (*this, "Open Scala (.scl) alternate tuning file...", FILE_CHOOSER_ACTION_OPEN);
+	dlg.add_button(Stock::CANCEL, RESPONSE_CANCEL);	dlg.add_button("Select", RESPONSE_OK);
+
+	FileFilter filter;
+	filter.set_name("Scala scale files");
+	filter.add_pattern("*.[Ss][Cc][Ll]");
+	dlg.add_filter(filter);
+
+	if (dlg.run() == RESPONSE_OK)
+	{
+		dlg.hide();
+		int error = vau->loadScale(dlg.get_filename());
+		if (error)
+		{
+			MessageDialog msg(*this, "Failed to load new tuning.");
+			msg.set_secondary_text("Reading the tuning file failed for some reason. \
+Make sure your file has the correct format and try again.");
+			msg.run();
+		}
+	}
+}
+
+void
+GUI::key_map_open	( )
+{
+	FileChooserDialog dlg (*this, "Open alternate keybord map (Scala .kbm format)...", FILE_CHOOSER_ACTION_OPEN);
+	dlg.add_button(Stock::CANCEL, RESPONSE_CANCEL);	dlg.add_button("Select", RESPONSE_OK);
+
+	FileFilter filter;
+	filter.set_name("Scala keyboard map files");
+	filter.add_pattern("*.[Kk][Bb][Mm]");
+	dlg.add_filter(filter);
+
+	if (dlg.run() == RESPONSE_OK)
+	{
+		dlg.hide();
+		int error = vau->loadKeyMap(dlg.get_filename());
+		if (error)
+		{
+			MessageDialog msg(*this, "Failed to load new keyboard map.");
+			msg.set_secondary_text("Reading the keyboard map file failed for some reason. \
+Make sure your file has the correct format and try again.");
+			msg.run();
+		}
+	}
+}
+
+void
+GUI::tuning_reset	( )
+{
+	MessageDialog dlg (*this, "Discard the current scale and keyboard map?");
+
+	if (dlg.run() == RESPONSE_OK)
+	{
+		vau->defaultTuning();
+	}
+}
+
 static gchar *which(gchar *command)
 {
 	gint exit_status = -1;
@@ -779,14 +961,170 @@ GUI::command_run	(const char *command)
 }
 
 void
-GUI::changed_midi_channel( )
+GUI::on_midi_channel_change(int value)
 {
-	if (midi_controller) midi_controller->set_midi_channel ((int)(adj_midi->get_value ()));
+	midi_controller->set_midi_channel(value);
 }
 
 void
-GUI::changed_voices	( )
+GUI::on_ployphony_change(int value)
 {
-	config->polyphony = (int)(adj_voices->get_value ());
-	if (vau) vau->SetMaxVoices (config->polyphony);
+	vau->SetMaxVoices(value);
 }
+
+#if ENABLE_MIDIKEYS
+
+////////////////////////////////////////////////////////////////////////////////
+//
+// Virtual Keyboard functionality
+//
+// Uses the computer's typing keyboard to play the synth.
+// Handy for testing and general playing with amSynth :)
+//
+//     W E   T Y U   O P
+//    A S D F G H J K L ; '
+//
+
+#define SIZEOF_ARRAY( a ) ( sizeof(a) / sizeof((a)[0]) )
+
+//
+// List of hardware keycodes for midi notes
+//
+// Raw hardware keycodes are used because we want the physical layout
+// to remain the same no matter what type of keyboard layout is used.
+//
+// Hardware keycodes seem to vary between platforms, although it's
+// difficult to find any decent documentation that covers this :-/
+//
+// This particular layout is based on that used by Logic Studio.
+//                                               
+static guint16 s_vkeybd_hardware_keycodes[] = {
+//	   A     W     S     E     D     F     T     G     Y     H     U     J     K     O     L     P     ;     '
+#if defined(__linux)
+	0x26, 0x19, 0x27, 0x1a, 0x28, 0x29, 0x1c, 0x2a, 0x1d, 0x2b, 0x1e, 0x2c, 0x2d, 0x20, 0x2e, 0x21, 0x2f, 0x30
+#elif defined(__APPLE__)
+	   8,   21,    9,   22,   10,   11,   25,   13,   24,   12,   40,   46,   48,   39,   45,   43,   49,   47
+#endif
+};
+
+//
+// returns -1 if hardware_keycode is not mapped to a midi note
+//
+static inline char midi_note_for_hardware_keycode( guint16 hardware_keycode )
+{
+	const guint16 kGreatestValidKeycode = 64;
+	static char value_for_code[kGreatestValidKeycode+1] = {0};
+	static bool initialised = false;
+	
+	// first-time initialisation (build the lookup table)
+	if (initialised == false) { initialised = true;
+		for (unsigned note=0; note<SIZEOF_ARRAY(s_vkeybd_hardware_keycodes); note++) {
+			guint16 code = s_vkeybd_hardware_keycodes[note];
+			value_for_code[code] = note + 1;
+		}
+	}
+	
+	if (hardware_keycode < kGreatestValidKeycode)
+		return value_for_code[hardware_keycode] - 1;
+
+	return -1;
+}
+
+bool GUI::on_key_press_event(GdkEventKey *inEvent)
+{
+	char midiNote = -1;
+
+//	fprintf(stderr, "string '%s' keyval 0x%02x keycode 0x%02x\n",
+//			inEvent->string,
+//			inEvent->keyval,
+//			inEvent->hardware_keycode);
+	
+	if (inEvent->keyval == GDK_Caps_Lock) {
+		if ((inEvent->state & GDK_LOCK_MASK) == GDK_LOCK_MASK) {
+			// will be disabled by this key press
+			m_vkeybdIsActive = false;
+			vkeybd_kill_all_notes();
+		} else {
+			m_vkeybdIsActive = true;
+		}
+		update_title();
+	}
+	
+	if ((inEvent->state & GDK_LOCK_MASK) == 0)
+		goto delegate;
+	
+	//
+	// switch between octaves using the number keys
+	//
+	switch (inEvent->keyval) {
+		case GDK_1: m_vkeybdOctave = 0; vkeybd_kill_all_notes(); return true;
+		case GDK_2: m_vkeybdOctave = 1; vkeybd_kill_all_notes(); return true;
+		case GDK_3: m_vkeybdOctave = 2; vkeybd_kill_all_notes(); return true;
+		case GDK_4: m_vkeybdOctave = 3; vkeybd_kill_all_notes(); return true;
+		case GDK_5: m_vkeybdOctave = 4; vkeybd_kill_all_notes(); return true;
+		case GDK_6: m_vkeybdOctave = 5; vkeybd_kill_all_notes(); return true;
+		case GDK_7: m_vkeybdOctave = 6; vkeybd_kill_all_notes(); return true;
+		case GDK_8: m_vkeybdOctave = 7; vkeybd_kill_all_notes(); return true;
+		case GDK_9: m_vkeybdOctave = 8; vkeybd_kill_all_notes(); return true;
+		case GDK_0: m_vkeybdOctave = 9; vkeybd_kill_all_notes(); return true;
+	}
+	
+	if ((midiNote = midi_note_for_hardware_keycode( inEvent->hardware_keycode )) == -1)
+		goto delegate;
+	
+	midiNote += (m_vkeybdOctave * 12);
+	if (m_vkeybdState[midiNote] == false) {
+		m_vkeybdState[midiNote]  = true;
+		vau->HandleMidiNoteOn(midiNote, 0.7f);
+	}
+	return true;
+	
+delegate:
+	return Gtk::Window::on_key_press_event(inEvent);
+}
+
+bool GUI::on_key_release_event(GdkEventKey *inEvent)
+{
+	char midiNote = -1;
+	
+	if ((inEvent->state & GDK_LOCK_MASK) != GDK_LOCK_MASK)
+		goto delegate;
+	
+	if ((midiNote = midi_note_for_hardware_keycode( inEvent->hardware_keycode )) == -1)
+		goto delegate;
+	
+#if defined(__linux)
+	// XkbSetDetectableAutoRepeat() seems to be broken on some Linux configurations,
+	// which means we can receive key release events for autorepeat. Therefore, we
+	// need to filter out these 'fake' key release events.
+	// http://bugs.freedesktop.org/show_bug.cgi?id=22515
+	{
+		char pressed_keys[32];
+		XQueryKeymap(gdk_x11_get_default_xdisplay(), pressed_keys);
+		bool isPressed = (pressed_keys[inEvent->hardware_keycode >> 3] >> (inEvent->hardware_keycode & 0x07)) & 0x01;
+		if (isPressed) {
+			goto delegate;
+		}
+	}
+#endif
+
+	midiNote += (m_vkeybdOctave * 12);
+	if (m_vkeybdState[midiNote] == true) {
+		m_vkeybdState[midiNote]  = false;
+		vau->HandleMidiNoteOff(midiNote, 0.7f);
+	}
+	return true;
+	
+delegate:
+	return Gtk::Window::on_key_release_event(inEvent);
+}
+
+void GUI::vkeybd_kill_all_notes()
+{
+	for (unsigned i=0; i<m_vkeybdState.size(); i++) {
+		vau->HandleMidiNoteOff(i, 0.0f);
+		m_vkeybdState[i] = false;
+	}
+}
+
+#endif
