@@ -8,8 +8,8 @@
 #include <string>
 #include <gtkmm.h>
 
+#include "../MidiController.h"
 #include "PresetControllerView.h"
-#include "ControllerMapDialog.h"
 #include "Request.h"
 
 class PresetController;
@@ -18,7 +18,6 @@ class Preset;
 class VoiceAllocationUnit;
 class GenericOutput;
 class Config;
-class EditorPanel;
 
 namespace Gtk {
 	class Adjustment;
@@ -61,13 +60,18 @@ public:
 
 protected:
 	virtual void	on_hide () { Gtk::Main::quit(); }
+	
+#if ENABLE_MIDIKEYS
+	virtual bool on_key_press_event(GdkEventKey *);
+	virtual bool on_key_release_event(GdkEventKey *);
+	void vkeybd_kill_all_notes();
+#endif
 
 private:		
 	Gtk::MenuBar*	create_menus		( );
 
 	int			lnav;
 	void		event_handler	(const int);
-	void config_controllers();
 	
 	void		preset_new		( );
 	void		preset_copy		( );
@@ -77,19 +81,26 @@ private:
 	void		bank_open		( );
 	void		bank_save_as		( );
 
+	void		scale_open		( );
+	void		key_map_open		( );
+	void		tuning_reset		( );
+
 	int		command_exists		(const char *command);
 	void		command_run		(const char *command);
 	
-	void		changed_midi_channel	( );
-	void		changed_voices		( );
+	void		on_midi_channel_change	(int value);
+	void		on_ployphony_change		(int value);
 	
 	void		post_init();
 	void		update_title();
 	void		UpdateParameterOnMainThread(Param, float);
+	
+	static void preset_paste_callback(GtkClipboard *clipboard, const gchar *text, gpointer data);
+	static void preset_paste_as_new_callback(GtkClipboard *clipboard, const gchar *text, gpointer data);
 
 	gint idle_callback();
 	gint setActiveParam( GdkEventButton *event, Parameter * param );
-	string status;
+	std::string status;
 	Gtk::VBox vbox;
 
 	Gtk::Style 		*style;
@@ -122,9 +133,6 @@ private:
 	Gtk::Statusbar		record_statusbar;
 	gboolean		record_recording;
 		
-	Gtk::Adjustment		*adj_midi,
-				*adj_voices;
-
 	Parameter *active_param;
 	PresetController *preset_controller;
 	PresetControllerView *presetCV;
@@ -132,15 +140,20 @@ private:
 	Config *config;
 	MidiController *midi_controller;
 	VoiceAllocationUnit *vau;
-	ControllerMapDialog *controller_map_dialog;
 	GenericOutput *audio_out;
 	
-	EditorPanel		*editor_panel;
-	
 	Preset			*clipboard_preset;
+	
+	GtkAdjustment 	*m_adjustments[kControls_End];
         
         std::string             m_baseName;
 	bool			m_presetIsNotSaved;
+
+#if ENABLE_MIDIKEYS
+	int					m_vkeybdOctave;
+	bool				m_vkeybdIsActive;
+	std::vector<bool>	m_vkeybdState;
+#endif
 };
 
 #endif
