@@ -13,13 +13,15 @@ using sigc::bind;
 using std::cout;
 using namespace std;
 
+extern Config config;
+
 PresetControllerView::PresetControllerView(VoiceAllocationUnit & vau )
 {
 	this->vau = &vau;
 	inhibit_combo_callback = false;
 	inhibit_combo_update = false;
 	
-    commit.add_label("Save Changes",0.5);
+    commit.add_label("Save",0.5);
     commit.signal_clicked().connect(
 		bind <const char*>(mem_fun(*this, &PresetControllerView::ev_handler),"commit"));
 
@@ -42,7 +44,7 @@ PresetControllerView::PresetControllerView(VoiceAllocationUnit & vau )
 	add(*aud);
 
 	Gtk::Button *panic = manage (new Gtk::Button);
-	panic->add_label ("Kill all voices");
+	panic->add_label ("Panic");
 	panic->signal_clicked().connect(bind(mem_fun(*this, &PresetControllerView::ev_handler),"panic"));
 	add (*panic);
 }
@@ -63,6 +65,7 @@ PresetControllerView::ev_handler(string text)
 {
 	if (text == "commit") {
 		presetController->commitPreset();
+		presetController->savePresets(config.current_bank_file.c_str());
 		update();
 		return;
 	} else if (text == "presets_combo") {
@@ -105,12 +108,9 @@ PresetControllerView::update()
 	}
 	
 	// set the display entries
-    char cstr[3];
-    sprintf(cstr, "%d", presetController->getCurrPresetNumber());
-	string txt("Preset ");
-	txt += string(cstr);
-	txt += " : ";
-    preset_no_entry.set_text(txt);	
-	
+	ostringstream oss;
+	oss << "Preset " << presetController->getCurrPresetNumber() << " : ";
+	preset_no_entry.set_text(oss.str());
+
 	inhibit_combo_callback = false;
 }
