@@ -1,5 +1,5 @@
 /* amSynth
- * (c) 2001-2005 Nick Dowell
+ * (c) 2001-2010 Nick Dowell
  */
 
 #ifndef _JACK_OUTPUT_H
@@ -12,11 +12,13 @@
 #include "AudioOutput.h"
 #include "Config.h"
 
+class MidiStreamReceiver;
+
 class JackOutput : public GenericOutput {
 
 public:
-				JackOutput	( );
-	virtual		~JackOutput	( ) {};
+
+	JackOutput();
 	
 	int			init		( Config & config );// returns 0 on success
 	bool		Start		();
@@ -28,27 +30,22 @@ public:
 	void		setOutputFile	( string file )	{ wavoutfile = file; };
 	string		getOutputFile	( )		{ return wavoutfile; };
 
-	const char*	getTitle	( )	{ return client_name.c_str(); };
+	void		setMidiHandler(MidiStreamReceiver *midiHandler) { _midiHandler = midiHandler; }
 
 #ifdef with_jack
-	int			process		(jack_nframes_t nframes);
-	int			bufsize		(jack_nframes_t nframes);
-	int			srate		(jack_nframes_t nframes);
-	void		shutdown	();
+	static int process(jack_nframes_t nframes, void *arg);
 #endif
 private:
 	int	running;
-	int	channels;
 	string	wavoutfile;
 	int	recording;
-	string	client_name, error_msg;
+	string	error_msg;
+	bool _auto_connect;
 #ifdef with_jack
-	jack_port_t 	*l_port, *r_port;
+	jack_port_t 	*l_port, *r_port, *m_port;
 	jack_client_t 	*client;
 #endif
-	int		sample_rate;
-	int		buf_size;
-	bool	initialised;
+	MidiStreamReceiver *_midiHandler;
 };
 
 #endif				// _JACK_OUTPUT_H
