@@ -15,7 +15,8 @@
 
 using namespace std;
 
-const int kMaxGrainSize = 64;
+const unsigned kMaxGrainSize = 64;
+const unsigned kBufferSize = 1024;
 
 static const VoiceBoardProcessMemory s_procMem (kMaxGrainSize);
 
@@ -24,19 +25,22 @@ VoiceAllocationUnit::VoiceAllocationUnit ()
 ,	mActiveVoices (0)
 ,	sustain (0)
 ,	mMasterVol (1.0)
+,	mPitchBendRangeSemitones(2)
 {
 	limiter = new SoftLimiter;
 	reverb = new revmodel;
 	distortion = new Distortion;
+	mBuffer = new float [kBufferSize * 2];
 
 	for (int i = 0; i < 128; i++)
 	{
 		keyPressed[i] = 0;
 		active[i] = false;
 		_voices.push_back (new VoiceBoard (&s_procMem));
-		_voices.back()->setFrequency ((440.0f/32.0f) * pow (2.0f, (float)((i-9.0)/12.0)));
 	}
 
+	updateTuning();
+
 	SetSampleRate (44100);
 }
 
@@ -46,6 +50,7 @@ VoiceAllocationUnit::~VoiceAllocationUnit	()
 	delete limiter;
 	delete reverb;
 	delete distortion;
+	delete [] mBuffer;
 }
 
 void
@@ -61,11 +66,9 @@ VoiceAllocationUnit::HandleMidiNoteOn(int note, float velocity)
 	assert (note >= 0);
 	assert (note < 128);
   
-	purgeVoices ();
-	
 	keyPressed[note] = 1;
 	
-	if ((!mMaxVoices || (mActiveVoices < mMaxVoices)) && !active[note])
+	if ((!mMaxVoices || (mActiveVoices < mMaxVoices)) && !active[note] && !mute[note])
 	{
 		_voices[note]->reset();
 		active[note]=1;
@@ -88,7 +91,7 @@ VoiceAllocationUnit::HandleMidiNoteOff(int note, float /*velocity*/)
 void
 VoiceAllocationUnit::HandleMidiPitchWheel(float value)
 {
-	float newval = pow(2.0f,value);
+	float newval = pow(2.0f, value * mPitchBendRangeSemitones / 12.0f);
 	for (unsigned i=0; i<_voices.size(); i++) _voices[i]->SetPitchBend (newval);
 }
 
@@ -125,8 +128,7 @@ void
 VoiceAllocationUnit::purgeVoices()
 {
 	for (unsigned note = 0; note < _voices.size(); note++) 
-		if (active[note] && (0 == _voices[note]->getState()))
-		{
+		if (active[note] && (_voices[note]->isSilent())) {
 			mActiveVoices--;
 			active[note] = 0;
 		}
@@ -136,21 +138,30 @@ VoiceAllocationUnit::purgeVoices()
 void
 VoiceAllocationUnit::Process		(float *l, float *r, unsigned nframes, int stride)
 {
-	// if stream is interleaved, do mono processing (non-interleaved) in the end part of the buffer.
-	float* vb = (1 < stride) ? l+nframes : l;
+	if (nframes > kBufferSize) {
+		this->Process(l,               r,                         kBufferSize, stride);
+		this->Process(l + kBufferSize, r + kBufferSize, nframes - kBufferSize, stride);
+		return;
+	}
+	
+	float* vb = mBuffer;
 	memset(vb, 0, nframes * sizeof (float));
 
-	int framesLeft=nframes; int j=0;
+	unsigned framesLeft = nframes, j = 0;
 	while (0 < framesLeft)
 	{
 		int fr = (framesLeft < kMaxGrainSize) ? framesLeft : kMaxGrainSize;
-		for (unsigned i=0; i<_voices.size(); i++) if (active[i]) _voices[i]->ProcessSamplesMix (vb+j, fr, mMasterVol);
+		for (unsigned i=0; i<_voices.size(); i++)
+			if (active[i] && !mute[i])
+				_voices[i]->ProcessSamplesMix (vb+j, fr, mMasterVol);
 		j += fr; framesLeft -= fr;
 	}
 
 	distortion->Process (vb, nframes);
 	reverb->processreplace (vb, l,r, nframes, 1, stride); // mono -> stereo
 	limiter->Process (l,r, nframes, stride);
+
+	purgeVoices();
 }
 
 void
@@ -168,3 +179,54 @@ VoiceAllocationUnit::UpdateParameter	(Param param, float value)
 	default: for (unsigned i=0; i<_voices.size(); i++) _voices[i]->UpdateParameter (param, value); break;
 	}
 }
+
+int
+VoiceAllocationUnit::loadScale		(const string & sclFileName)
+{
+	int error = tuningMap.loadScale(sclFileName);
+	if (error)
+		return error;
+	else
+	{
+		updateTuning();
+		return 0;
+	}
+}
+
+int
+VoiceAllocationUnit::loadKeyMap		(const string & kbmFileName)
+{
+	int error = tuningMap.loadKeyMap(kbmFileName);
+	if (error)
+		return error;
+	else
+	{
+		updateTuning();
+		return 0;
+	}
+}
+
+void
+VoiceAllocationUnit::defaultTuning	()
+{
+	tuningMap.defaultScale();
+	tuningMap.defaultKeyMap();
+	updateTuning();
+}
+
+void
+VoiceAllocationUnit::updateTuning	()
+{
+	for (int i = 0; i < 128; i++)
+	{
+		double pitch = tuningMap.noteToPitch(i);
+		if (pitch < 0)
+			mute[i] = true; // unmapped key
+		else
+		{
+			_voices[i]->setFrequency(pitch);
+			mute[i] = false;
+		}
+	}
+}
+
